@@ -101,13 +101,19 @@ def _split_path_into_dirs(path):
 
 
 class ProjectImage(object):
+    '''
+    Right now this class supports two types of image structures.
+    1. images/chap1-image1.png
+    2. images/chap1/image1.png
+    '''
+
     def __init__(self, rel_path, parent_filename, image_dir):
         self.parent_filename = parent_filename
-        (head, tail) = os.path.splitext(parent_filename)
+        (parent_head, parent_tail) = os.path.splitext(parent_filename)
         # chap1.re -> 'chap1'
-        self.parent_id = head
+        self.parent_id = parent_head
         # '.re'
-        self.parent_tail = tail
+        self.parent_tail = parent_tail
         # e.g. 'images/chap1-image1.png'
         self.rel_path = rel_path
         self.image_dir = image_dir
@@ -119,16 +125,18 @@ class ProjectImage(object):
         # 'chap1-image1.png' -> ('chap1-image1', '.png')
 
         if len(parts) == 3:
-            assert head == self.parent_id
+            # ['images', 'chap1', 'image1.png']
+            assert parts[1] == self.parent_id
             (head, tail) = os.path.splitext(parts[2])
             self.id = head
-        else:
+        else:  # len(parts) == 2
             (head, tail) = os.path.splitext(parts[1])
             # e.g. 'chap1-image1' should start with 'chap1-'
             assert head.startswith('{}-'.format(self.parent_id))
             # e.g. 'images/chap1-image1.png' -> image1
             self.id = head[len(self.parent_id)+1:]
         assert self.id
+        # '.png'
         self.tail = tail
 
     def __str__(self):
@@ -179,26 +187,36 @@ class ReVIEWProject(object):
     # True if column. False (or None) otherwise.
     BM_IS_COLUMN = 'is_column'
 
-    def __init__(self, source_dir, **kwargs):
-        self.reset()
-        self.init(os.path.normpath(source_dir), **kwargs)
+    @staticmethod
+    def instantiate(source_dir, **kwargs):
+        driver = ReVIEWProject(source_dir,
+                               logger=kwargs.get('logger'))
+        if driver.init():
+            return driver
+        else:
+            return None
 
-    def reset(self):
+    def __init__(self, source_dir, logger=None):
         # Where the whole source files are.
-        self.source_dir = None
+        self.source_dir = os.path.normpath(source_dir)
+        self.logger = logger or local_logger
+        self._reset()
+
+    def _reset(self):
         self._catalog_files = []
         # Contains all chapter files for this project.
-        # This includes predef/postdef files.
         # Each name does not contain source_dir part.
+        # This includes predef/postdef files,
+        # while it does NOT include possible draft filenames.
         self.source_filenames = []
 
         self.predef_filenames = []
         self.postdef_filenames = []
 
-        # Possible "draft" files that are not included in catalog.yml
-        # but in the source_dir.
-        # TODO:-)
-        # self.draft_filenames = []
+        # Possible "draft" files.
+        # The "draft" files are re files which are in source_dir, but
+        # not included in catalog.yml.
+        self.draft_filenames = []
 
         # Either self.parts or self.chaps is available. NOT both.
         # When PART (in catalog.yml or as a single file) is available,
@@ -214,6 +232,8 @@ class ReVIEWProject(object):
         self.chaps = None
 
         self.image_dir = None
+        # Contains a mapping from filenames to images relevant to the files.
+        # This will include all mapping including draft filenames.
         # {'chap1.re': [ProjectImage, ...]}
         self.images = {}
 
@@ -242,17 +262,14 @@ class ReVIEWProject(object):
         # Not ready until init() is finished successfully.
         self.ready = False
 
-    def init(self, source_dir, **kwargs):
+    def init(self, **kwargs):
         '''
-        Initializes this instance. Returns True when successful.
-
+        Initializes this instance.
         Returns True when successful.
         Returns False otherwise, where this instance will not be usable.
         '''
-        logger = kwargs.get('logger') or local_logger
-        self.logger = logger
-        self.source_dir = source_dir
-        logger.debug(u'init(source_dir: "{}")'.format(self.source_dir))
+        logger = kwargs.get('logger') or self.logger
+        logger.debug(u'ReVIEWProject.init()')
 
         first_candidate = kwargs.get('first_candidate')
         if not self._find_and_parse_review_config(first_candidate):
@@ -265,20 +282,24 @@ class ReVIEWProject(object):
 
         # Look for "catalog.yml" or legacy old catalog files.
         self._recognize_catalog_files()
-        if (not self.parts) and (not self.chaps):
-            self.logger.info(u'Failed to recognize book structure in {}.'
-                             .format(source_dir))
+        if (self.parts is None) and (self.chaps is None):
+            self.logger.warn(u'Failed to recognize book structure in {}.'
+                             .format(self.source_dir))
             return False
-        assert (not self.parts) or (not self.chaps)
+        if (not self.parts) and (not self.parts):
+            self.logger.info(u'No chapter found.')
+
+        self._recognize_draft_files()
 
         self.image_dir = kwargs.get('image_dir', 'images')
         self.image_dir_path = os.path.normpath('{}/{}'.format(self.source_dir,
                                                               self.image_dir))
-        if not os.path.isdir(self.image_dir_path):
+        self.images = {}
+        if os.path.isdir(self.image_dir_path):
+            self._recognize_image_files()
+        else:
             self.logger.info(u'"{}"({}) is not a directory'
                              .format(self.image_dir, self.image_dir_path))
-        self.images = {}
-        self._recognize_image_files()
 
         # TODO: Check more..
 
@@ -311,7 +332,6 @@ class ReVIEWProject(object):
         check if it is really an appropriate config for Re:VIEW.
         If it looks appropriate, set up member variables too.
         '''
-        self.logger.debug(u'_try_parse_review_config({})'.format(candidate))
         candidate_path = os.path.join(self.source_dir, candidate)
         if not os.path.isfile(candidate_path):
             return False
@@ -333,12 +353,10 @@ class ReVIEWProject(object):
                              .format(candidate, e))
         return False
 
-
     def _recognize_catalog_files(self):
         '''
         Scans catalogue files and detect book structure.
         '''
-        self.logger.debug(u'_recognize_catalog_files()')
         if self._recognize_new_catalog_files():
             return True
         return self._recognize_legacy_catalog_files()
@@ -409,7 +427,7 @@ class ReVIEWProject(object):
                 for filename in part_chaps:
                     self.source_filenames.append(filename)
         else:
-            logger.debug('Considered to be plain chaps structure')
+            logger.debug('Considered to be plain chaps without part.')
             self.chaps = []
             self.parts = None
             try:
@@ -442,7 +460,6 @@ class ReVIEWProject(object):
         which has been used before Re:VIEW version 1.3.
         '''
         logger = self.logger
-        self.logger.debug('_recognize_legacy_catalog_files()')
         # First check if at least "CHAPS" file exists or not.
         # If not, abort this procedure immediately.
         chaps_path = _verify_filename(self.source_dir, 'CHAPS', logger)
@@ -532,7 +549,6 @@ class ReVIEWProject(object):
                     continue
                 self.postdef_filenames.append(filename)
                 self.source_filenames.append(filename)
-
         return True
         
     def _detect_parts(self, part_content):
@@ -541,6 +557,14 @@ class ReVIEWProject(object):
             part_titles.append(line.rstrip())
         return part_titles
 
+    def _recognize_draft_files(self):
+        logger = self.logger
+        for re_file in filter(lambda x: x.endswith('.re'),
+                              os.listdir(self.source_dir)):
+            if re_file not in self.source_filenames:
+                self.draft_filenames.append(re_file)
+        return True
+
     def parse_source_files(self, logger=None):
         '''
         Parsees all Re:VIEW files and prepare internal structure.
@@ -618,6 +642,7 @@ class ReVIEWProject(object):
         Removes possible tempfiles from the project.
         '''
         logger = logger or self.logger
+        logger.debug('ReVIEWProject.remove_tempfiles()')
         bookname = self.bookname or u'book'
         temp_dirs = map(lambda x: x.format(bookname),
                         [u'{}', u'{}-pdf', u'{}-epub', u'{}-log'])
@@ -631,12 +656,21 @@ class ReVIEWProject(object):
         else:
             return False
 
+    def has_source(self, re_file):
+        return re_file in self.all_filenames()
+
+    def get_images_for_source(self, re_file):
+        return self.images.get(re_file, [])
+
+    def all_filenames(self):
+        return self.source_filenames + self.draft_filenames
+
     def _recognize_image_files(self):
         if not os.path.isdir(self.image_dir_path):
             self.logger.debug(u'No image_dir ("{}")'
                               .format(self.image_dir_path))
             return
-        parent_filenames = sorted(self.source_filenames)
+        parent_filenames = sorted(self.all_filenames())
         image_filenames = sorted(os.listdir(self.image_dir_path))
         i_parents = 0
         i_images = 0
@@ -649,12 +683,13 @@ class ReVIEWProject(object):
             # e.g. 'chap1-test1.png', 'chap1/test1.png'
             image_filename = image_filenames[i_images]
             rel_path = '{}/{}'.format(self.image_dir, image_filename)
+            abs_path = os.path.normpath('{}/{}'.format(self.image_dir_path,
+                                                       image_filename))
             (head, tail) = os.path.splitext(image_filename)
-
-            if os.path.isdir(rel_path):
+            if os.path.isdir(abs_path):
                 # e.g. rel_path ... 'images/chap1/test1.png'
                 if parent_id == image_filename:
-                    for image_filename2 in os.listdir(rel_path):
+                    for image_filename2 in os.listdir(abs_path):
                         rel_path2 = '{}/{}'.format(rel_path, image_filename2)
                         pi = ProjectImage(rel_path=rel_path2,
                                           parent_filename=parent_filename,
@@ -708,6 +743,8 @@ class ReVIEWProject(object):
         logger.debug(u'postdef_filenames(len: {}): {}'
                      .format(len(self.postdef_filenames),
                              self.postdef_filenames))
+        logger.debug(u'draft_filenames(len: {}): {}'
+                     .format(len(self.draft_filenames), self.draft_filenames))
         if self.parts:
             logger.debug(u'parts: {}'.format(self.parts))
         elif self.chaps:
